@@ -138,11 +138,12 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
   static const char *protocol_str[] =
     { "http", "https", "ftp", "file", "mms", "unknown" };
   int protocol = PROTOCOL_HTTP;
-  char *normadr;
-  char *normfil;
-  char *fil;
-  char *adr;
-  char *print_adr;
+  const char *const adr = jump_identification(adr_complete);
+  char *fil = fil_complete;
+  // copy of fil, used for lookups (see urlhack)
+  const char *normadr = adr;
+  const char *normfil = fil;
+  const char *const print_adr = jump_protocol(adr);
   char *start_pos = NULL, *nom_pos = NULL, *dot_pos = NULL;     // Position nom et point
 
   // pour changement d'extension ou de nom (content-disposition)
@@ -153,6 +154,7 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
 
   //CLEAR
   newfil[0] = ext[0] = '\0';
+  save[0] = '\0';
 
   /* 8-3 ? */
   switch (opt->savename_83) {
@@ -167,20 +169,6 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
     break;
   }
 
-  // effacer save
-  save[0] = '\0';
-  // fil
-  fil = fil_complete;
-  // copy of fil, used for lookups (see urlhack)
-  normfil = fil;
-  // et adr (sauter user/pass)
-  // on prend le parti de mettre les fichiers avec login/pass au même endroit que si ils
-  // étaient capturés sans ces paramètres
-  // c'est pour cette raison qu'on ignore totalement adr_complete (même pour la recherche en table de hachage)
-  adr = jump_identification(adr_complete);
-  // copy of adr, used for lookups (see urlhack)
-  normadr = adr;
-
   // normalize the URL:
   // www.foo.com -> foo.com
   // www-42.foo.com -> foo.com
@@ -204,7 +192,6 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
   }
 
   // à afficher sans ftp://
-  print_adr = jump_protocol(adr);
   if (strfield(adr_complete, "https:")) {
     protocol = PROTOCOL_HTTPS;
   } else if (strfield(adr_complete, "ftp:")) {
@@ -225,6 +212,29 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
     }
   }
 
+  /* Declare adr (IDNA-decoded if necessary) */
+#define DECLARE_ADR(FINAL_ADR) \
+  char *idna_adr =\
+    /* http or https */\
+    (\
+    protocol == PROTOCOL_HTTP\
+    || protocol == PROTOCOL_HTTPS \
+    )\
+    /* and contains IDNA */\
+    && hts_isStringIDNA(adr_complete, strlen(print_adr))\
+    ? hts_convertStringIDNAToUTF8(print_adr, strlen(print_adr))\
+    : NULL;\
+  const char *const FINAL_ADR = idna_adr != NULL \
+    ? idna_adr : ( protocol == PROTOCOL_FILE ? "file" : print_adr )
+
+  /* Release adr */
+#define RELEASE_ADR() do {\
+  if (idna_adr != NULL) {\
+    free(idna_adr);\
+    idna_adr = NULL;\
+  }\
+} while(0)
+
   // vérifier que le nom n'a pas déja été calculé (si oui le renvoyer tel que)
   // vérifier que le nom n'est pas déja pris...
   // NOTE: si on cherche /toto/ et que /toto est trouvé on le prend (et réciproquqment) ** // **
@@ -349,10 +359,10 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
   if (is_html < 0 && opt->check_type && !ext_chg) {
     int ishtest = 0;
 
-    if ((!strfield(adr_complete, "file://"))
-        && (!strfield(adr_complete, "ftp://"))
+    if (protocol != PROTOCOL_FILE
+        && protocol != PROTOCOL_FTP
 #if HTS_USEMMS
-        && (!strfield(adr_complete, "mms://"))
+        && protocol != PROTOCOL_MMS
 #endif
       ) {
       // tester type avec requète HEAD si on ne connait pas le type du fichier
@@ -937,9 +947,25 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
           }
           b += strlen(b);       // pointer à la fin
           break;
-        case 'h':              // host
+        case 'h':              // host (IDNA decoded if suitable)
+          // IDNA / RFC 3492 (Punycode) handling for HTTP(s)
+          {
+            DECLARE_ADR(final_adr);
+
+            /* Copy address */
+            *b = '\0';
+            if (!short_ver)
+              strcpybuff(b, final_adr);
+            else
+              strcpybuff(b, final_adr);
+
+            /* release */
+            RELEASE_ADR();
+          }
+          break;
+        case 'H':              // host, raw (old mode)
           *b = '\0';
-          if (strcmp(adr_complete, "file://") == 0) {
+          if (protocol == PROTOCOL_FILE) {
             if (!short_ver)     // Noms longs
               strcpybuff(b, "localhost");
             else
@@ -1012,26 +1038,24 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
     /* recopier www.. */
     if (opt->savename_type != 100) {
       if (((opt->savename_type / 1000) % 2) == 0) {     // >1000 signifie "pas de www/"
-        if (strcmp(adr_complete, "file://") == 0) {
-          //## if (*adr==lOCAL_CHAR) {
-          if (opt->savename_83 != 1)    // noms longs
-            strcatbuff(save, "localhost");
-          else
-            strcatbuff(save, "local");
-        } else {
-          // adresse url
-          if (!opt->savename_83) {      // noms longs (et pas de .)
-            strcatbuff(save, print_adr);
-          } else {              // noms 8-3
-            if (strlen(print_adr) > 4) {
-              if (strfield(print_adr, "www."))
-                strncatbuff(save, print_adr + 4, max_char);
-              else
-                strncatbuff(save, print_adr, 8);
-            } else
-              strncatbuff(save, print_adr, max_char);
-          }
+        DECLARE_ADR(final_adr);
+
+        // adresse url
+        if (!opt->savename_83) {      // noms longs (et pas de .)
+          strcatbuff(save, final_adr);
+        } else {              // noms 8-3
+          if (strlen(final_adr) > 4) {
+            if (strfield(final_adr, "www."))
+              hts_appendStringUTF8(save, final_adr + 4, max_char);
+            else
+              hts_appendStringUTF8(save, final_adr, max_char);
+          } else
+            hts_appendStringUTF8(save, final_adr, max_char);
         }
+
+        /* release */
+        RELEASE_ADR();
+
         if (*fil != '/')
           strcatbuff(save, "/");
       }
@@ -1060,30 +1084,26 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
     // dossier "web" ou "www.xxx" ?
     if (((opt->savename_type / 1000) % 2) == 0) {       // >1000 signifie "pas de www/"
       if ((opt->savename_type / 100) % 2) {
-        if (strcmp(adr_complete, "file://") == 0) {
-          //## if (*adr==lOCAL_CHAR) {
-          if (opt->savename_83 != 1)    // noms longs
-            strcatbuff(save, "localhost/");
-          else
-            strcatbuff(save, "local/");
-        } else {
-          // adresse url
-          if (!opt->savename_83) {      // noms longs
-            strcatbuff(save, print_adr);
+        DECLARE_ADR(final_adr);
+
+        if (!opt->savename_83) {      // noms longs
+          strcatbuff(save, final_adr);
+          strcatbuff(save, "/");
+        } else {              // noms 8-3
+          if (strlen(final_adr) > 4) {
+            if (strfield(final_adr, "www."))
+              hts_appendStringUTF8(save, final_adr + 4, max_char);
+            else
+              hts_appendStringUTF8(save, final_adr, max_char);
+            strcatbuff(save, "/");
+          } else {
+            hts_appendStringUTF8(save, final_adr, max_char);
             strcatbuff(save, "/");
-          } else {              // noms 8-3
-            if (strlen(print_adr) > 4) {
-              if (strfield(print_adr, "www."))
-                strncatbuff(save, print_adr + 4, max_char);
-              else
-                strncatbuff(save, print_adr, max_char);
-              strcatbuff(save, "/");
-            } else {
-              strncatbuff(save, print_adr, max_char);
-              strcatbuff(save, "/");
-            }
           }
         }
+
+        /* release */
+        RELEASE_ADR();
       } else {
         strcatbuff(save, "web/");       // répertoire général
       }
@@ -1261,16 +1281,15 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
   }
   //
   {                             // éliminer les // (comme ftp://)
-    char *a;
+    size_t i;
 
-    //while( (a=strstr(save,"//")) ) *a='_';
     cleanDoubleSlash(save);
-    // Eliminer chars spéciaux
-    a = save - 1;
-    while(*(++a))
-      if (((unsigned char) (*a) <= 31)
-          || ((unsigned char) (*a) == 127))
-        *a = '_';
+
+    for(i = 0 ; save[i] != '\0' ; i++) {
+      unsigned char c = (unsigned char) save[i];
+      if (c < 32 || c == 127)
+        save[i] = '_';
+    }
   }
 
 #if HTS_OVERRIDE_DOS_FOLDERS
@@ -1399,54 +1418,89 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
       strcatbuff(lastDot, "." DELAYED_EXT);
     }
   }
-  // enforce 256-character path limit before inserting destination path
-#define HTS_MAX_PATH_LEN 250
-#define MIN_LAST_SEG_RESERVE 30
+  // enforce 260-character path limit before inserting destination path
+  // note: 12 characters at least for WIN32, and 12 for ".99.delayed"
+  // (MSDN) "When using an API to create a directory, the specified path 
+  // cannot be so long that you cannot append an 8.3 file name 
+  // (that is, the directory name cannot exceed MAX_PATH minus 12)."
+#define HTS_MAX_PATH_LEN ( 260 - 12 - 12 )
+#define MIN_LAST_SEG_RESERVE 12
+#define MAX_LAST_SEG_RESERVE 24
+#define MAX_SEG_LEN 48
   if (hts_stringLengthUTF8(save) +
       hts_stringLengthUTF8(StringBuff(opt->path_html_utf8)) >=
       HTS_MAX_PATH_LEN) {
-    char BIGSTK tempo[HTS_URLMAXSIZE * 2];
-    const size_t parentLen =
-      hts_stringLengthUTF8(StringBuff(opt->path_html_utf8));
-    // parent path length is not insane (otherwise, ignore and pick 200 as suffix length)
-    const size_t maxLen =
-      parentLen <
-      HTS_MAX_PATH_LEN / 2 ? HTS_MAX_PATH_LEN - parentLen : HTS_MAX_PATH_LEN;
-    size_t i, j, lastSeg, sofar;
+    // convert to Unicode (much simpler)
+    size_t wsaveLen;
+    hts_UCS4 *const wsave = hts_convertUTF8StringToUCS4(save, strlen(save), &wsaveLen);
+    if (wsave != NULL) {
+      const size_t parentLen =
+        hts_stringLengthUTF8(StringBuff(opt->path_html_utf8));
+      // parent path length is not insane (otherwise, ignore and pick 200 as 
+      // suffix length)
+      const size_t maxLen =
+        parentLen <
+        HTS_MAX_PATH_LEN - HTS_MAX_PATH_LEN / 4
+        ? HTS_MAX_PATH_LEN - parentLen : HTS_MAX_PATH_LEN;
+      size_t i, j, lastSeg, lastSegSize, dirSize;
+      char *saveFinal;
 
-    // pick up last segment
-    for(i = 0, lastSeg = 0; save[i] != '\0'; i++) {
-      if (save[i] == '/') {
-        lastSeg = i + 1;
+      // pick up last segment
+      for(i = 0, lastSeg = 0; wsave[i] != '\0'; i++) {
+        if (wsave[i] == '/') {
+          lastSeg = i + 1;
+        }
       }
-    }
-    // add as much pathes as we can
-    for(i = 0, sofar = 0; i < lastSeg && i + MIN_LAST_SEG_RESERVE < maxLen; i++) {
-      tempo[i] = save[i];
-      if (save[i] == '/') {
-        // validate segment so far
-        sofar = i + 1;
+      lastSegSize = wsaveLen - lastSeg;
+      if (lastSegSize > MAX_LAST_SEG_RESERVE) {
+        lastSegSize = MAX_LAST_SEG_RESERVE;
       }
-    }
-    // last segment
+      else if (lastSegSize < MIN_LAST_SEG_RESERVE) {
+        lastSegSize = MIN_LAST_SEG_RESERVE;
+      }
+
+      // add as much pathes as we can.
+      // note: i is in bytes, iUtf in characters
+      for(i = 0, j = 0, dirSize = 0
+        ; i + 1 < lastSeg && j + lastSegSize < maxLen; i++) {
+          // reset segment counting
+          if (wsave[i] == '/') {
+            dirSize = 0;
+          }
+
+          // copy if not too long
+          if (dirSize < MAX_SEG_LEN) {
+            wsave[j++] = wsave[i];
+            dirSize++;
+          }
+      }
+
+      // last segment
+      wsave[j++] = '/';
 #define MAX_UTF8_SEQ_CHARS 4
-    for(j = 0; save[j + i] != '\0'; j++) {
-      // Stop here before next sequence
-      if (sofar + j + MAX_UTF8_SEQ_CHARS >= maxLen
-          && HTS_IS_LEADING_UTF8(save[i + j])) {
-        break;
+      for(i = lastSeg; wsave[i] != '\0' && j < maxLen; i++) {
+        wsave[j++] = wsave[i];
       }
-      // Stop is overflowing
-      else if (sofar + j >= maxLen) {
-        break;
+      // terminating \0
+      wsave[j++] = '\0';
+
+      // copy final name and cleanup
+      saveFinal = hts_convertUCS4StringToUTF8(wsave, j);
+      if (saveFinal != NULL) {
+        strcpybuff(save, saveFinal);
+        free(saveFinal);
+      } else {
+        hts_log_print(opt, LOG_ERROR, "Could not revert to UTF-8: %s%s",
+          adr_complete, fil_complete);
       }
-      save[sofar + j] = save[i + j];
+      free(wsave);
+
+      // log in debug
+      hts_log_print(opt, LOG_DEBUG, "Too long filename shortened: %s%s => %s",
+        adr_complete, fil_complete, save);
+    } else {
+      hts_log_print(opt, LOG_ERROR, "Could not read UTF-8: %s", save);
     }
-    // terminating \0
-    save[sofar + j] = '\0';
-    // log in debug
-    hts_log_print(opt, LOG_DEBUG, "Too long filename shortened: %s%s => %s",
-                  adr_complete, fil_complete, save);
   }
 #undef MAX_UTF8_SEQ_CHARS
 #undef MIN_LAST_SEG_RESERVE
@@ -1469,7 +1523,8 @@ int url_savename(char *adr_complete, char *fil_complete, char *save,
 
       //
       nom_ok = 1;               // à priori bon
-      // on part de la fin pour optimiser, plus les opti de taille pour aller encore plus vite..
+      // on part de la fin pour optimiser, plus les opti de taille pour 
+      // aller encore plus vite..
 #if DEBUG_SAVENAME
       printf("\nStart search\n");
 #endif
